package fr.insa.clavardator.client.users;

import fr.insa.clavardator.lib.network.TcpConnection;
import fr.insa.clavardator.lib.util.ErrorCallback;
import fr.insa.clavardator.lib.util.Log;
import fr.insa.clavardator.lib.util.SimpleCallback;
import org.jetbrains.annotations.Nullable;

import java.io.EOFException;
import java.io.Serializable;

public class DirectPeerConnection extends PeerConnection {
	private final TcpConnection connection;

	public DirectPeerConnection(TcpConnection connection, PeerUser user) {
		super(user);
		this.connection = connection;
	}

	@Override
	protected void send(Serializable message, SimpleCallback callback, @Nullable ErrorCallback errorCallback) {
		connection.send(message, callback, errorCallback);
	}


	@Override
	protected void receive(TcpConnection.MessageReceivedCallback callback, ErrorCallback errorCallback) {
		connection.receive(callback,
				e -> {
					disconnect();
					if (!(e instanceof EOFException)) {
						Log.e(this.getClass().getSimpleName(), "Error receiving message from " + user.getId(), e);
						errorCallback.onError(e);
					}
				});
	}


	/**
	 * Close the connection to this user
	 */
	private void closeConnection() {
		if (connection != null && connection.isOpen()) {
			connection.close();
		}
	}

	@Override
	public void disconnect() {
		closeConnection();
	}
}
