package fr.insa.clavardator.client.users;

import fr.insa.clavardator.client.server.Proxy;
import fr.insa.clavardator.lib.network.TcpConnection;
import fr.insa.clavardator.lib.util.ErrorCallback;
import fr.insa.clavardator.lib.util.Log;
import fr.insa.clavardator.lib.util.SimpleCallback;
import org.jetbrains.annotations.Nullable;

import java.io.EOFException;
import java.io.Serializable;

public class ProxyPeerConnection extends PeerConnection {
	private final Proxy proxy;

	public ProxyPeerConnection(Proxy proxy, PeerUser user) {
		super(user);
		this.proxy = proxy;
	}

	@Override
	protected void send(Serializable message, SimpleCallback callback, @Nullable ErrorCallback errorCallback) {
		proxy.send(message, callback, errorCallback);
	}


	@Override
	protected void receive(TcpConnection.MessageReceivedCallback callback, ErrorCallback errorCallback) {
		proxy.receive(user.getId(), callback,
				e -> {
					disconnect();
					if (!(e instanceof EOFException)) {
						Log.e(this.getClass().getSimpleName(), "Error receiving message from " + user.getId(), e);
						errorCallback.onError(e);
					}
				});
	}

	@Override
	public void disconnect() {
		proxy.disconnectUser(user.getId());
	}
}
